@@ -22,7 +22,8 @@ module Database.Sqlite  (
                          bindText,
                          bind,
                          column,
-                         columns
+                         columns,
+                         changes
                         )
     where
 
@@ -33,13 +34,15 @@ import qualified Data.ByteString as BS
 import qualified Data.ByteString.Internal as BSI
 import Foreign
 import Foreign.C
-import Database.Persist.Store (PersistValue (..), listToJSON, mapToJSON)
+import Database.Persist.Store (PersistValue (..), listToJSON, mapToJSON, ZT (ZT))
 import Data.Text (Text, pack, unpack)
 import Data.Text.Encoding (encodeUtf8, decodeUtf8With)
 import Data.Text.Encoding.Error (lenientDecode)
 import Data.Monoid (mappend, mconcat)
+import Data.IORef (IORef, newIORef, readIORef, writeIORef)
 
-newtype Connection = Connection (Ptr ())
+data Connection = Connection !(IORef Bool) Connection'
+newtype Connection' = Connection' (Ptr ())
 newtype Statement = Statement (Ptr ())
 
 data Error = ErrorOK
@@ -125,7 +128,7 @@ decodeColumnType i = Prelude.error $ "decodeColumnType " ++ show i
 foreign import ccall "sqlite3_errmsg"
   errmsgC :: Ptr () -> IO CString
 errmsg :: Connection -> IO Text
-errmsg (Connection database) = do
+errmsg (Connection _ (Connection' database)) = do
   message <- errmsgC database
   byteString <- BS.packCString message
   return $ decodeUtf8With lenientDecode byteString
@@ -157,7 +160,8 @@ openError path' = do
                                case error of
                                  ErrorOK -> do
                                             database' <- peek database
-                                            return $ Left $ Connection database'
+                                            active <- newIORef True
+                                            return $ Left $ Connection active $ Connection' database'
                                  _ -> return $ Right error))
 open :: Text -> IO Connection
 open path = do
@@ -169,7 +173,8 @@ open path = do
 foreign import ccall "sqlite3_close"
   closeC :: Ptr () -> IO Int
 closeError :: Connection -> IO Error
-closeError (Connection database) = do
+closeError (Connection iactive (Connection' database)) = do
+  writeIORef iactive False
   error <- closeC database
   return $ decodeError error
 close :: Connection -> IO ()
@@ -182,7 +187,7 @@ close database = do
 foreign import ccall "sqlite3_prepare_v2"
   prepareC :: Ptr () -> CString -> Int -> Ptr (Ptr ()) -> Ptr (Ptr ()) -> IO Int
 prepareError :: Connection -> Text -> IO (Either Statement Error)
-prepareError (Connection database) text' = do
+prepareError (Connection _ (Connection' database)) text' = do
   BS.useAsCString (encodeUtf8 text')
                   (\text -> do
                      alloca (\statement -> do
@@ -220,12 +225,16 @@ resetError :: Statement -> IO Error
 resetError (Statement statement) = do
   error <- resetC statement
   return $ decodeError error
-reset :: Statement -> IO ()
-reset statement = do
-  error <- resetError statement
-  case error of
-    ErrorOK -> return ()
-    _ -> return () -- FIXME confirm this is correct sqlError Nothing "reset" error
+reset :: Connection -> Statement -> IO ()
+reset (Connection iactive _) statement = do
+  active <- readIORef iactive
+  if active
+      then do
+          error <- resetError statement
+          case error of
+            ErrorOK -> return ()
+            _ -> return () -- FIXME confirm this is correct sqlError Nothing "reset" error
+      else return ()
 
 foreign import ccall "sqlite3_finalize"
   finalizeC :: Ptr () -> IO Int
@@ -341,6 +350,7 @@ bind statement sqlData = do
             PersistDay d -> bindText statement parameterIndex $ pack $ show d
             PersistTimeOfDay d -> bindText statement parameterIndex $ pack $ show d
             PersistUTCTime d -> bindText statement parameterIndex $ pack $ show d
+            PersistZonedTime (ZT d) -> bindText statement parameterIndex $ pack $ show d
             PersistList l -> bindText statement parameterIndex $ listToJSON l
             PersistMap m -> bindText statement parameterIndex $ mapToJSON m
             PersistObjectId _ -> P.error "Refusing to serialize a PersistObjectId to a SQLite value"
@@ -417,3 +427,9 @@ columns :: Statement -> IO [PersistValue]
 columns statement = do
   count <- columnCount statement
   mapM (\i -> column statement i) [0..count-1]
+
+foreign import ccall "sqlite3_changes"
+  changesC :: Connection' -> IO Int
+
+changes :: Connection -> IO Int64
+changes (Connection _ c) = fmap fromIntegral $ changesC c
